﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TIS.Imaging;
using NAudio.MediaFoundation;

namespace Memory_recording
{
    public partial class Form1 : Form
    {
        // この属性は、フォームのIC ImagingControlウィンドウのサイズに一致するように
        // ライブディスプレイを拡大縮小するかどうかを制御します。
        private bool m_bFitImageToWindow = true;


        private SinkListener _sinkListener = new SinkListener();
        FrameQueueSink _sink;
     
        // キャプチャする時間（テキストボックスから取得します。）
        private int _FramesToCapture = 0;

        // _bufferlistには、キャプチャされた画像のリストが含まれます。 
        // このリストの画像は画像ボックスに表示されます。
        private IFrameQueueBuffer [] _bufferlist;

        private ulong _availablePhysicalMemory; //合計物理メモリ
        public Form1()
        {
            MediaFoundationApi.Startup();

            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // 以前に使用したデバイスを読み込み
            try
            {
                icImagingControl1.LoadDeviceStateFromFile("device.xml", true);
                GetFramerateLabel();
            }
            catch
            {
                //xmlファイルが存在しないか、デバイスをロードできませんでした。 
                //どちらの場合も、何もせずに続行します。
            }

            if (m_bFitImageToWindow)
            {
                // ライブディスプレイのサイズをコントロールサイズに調整します。
                icImagingControl1.LiveDisplayDefault = false;
                icImagingControl1.LiveDisplaySize = icImagingControl1.Size;
            }

            // _sinkListenerに基づいてシンクを作成します。 [準備]ボタンがクリックされた場合、
            // _sinkはバッファキューを渡します。
            // したがって、クラスにはシンクのメンバーが必要です。
            _sink = new FrameQueueSink(_sinkListener, MediaSubtypes.RGB32);
            icImagingControl1.Sink = _sink;
                       
            UpdateControls();

        }

        /// <summary>
        /// アプリケーションが閉じている場合は、ライブビデオストリームを停止します。
        /// </summary>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (icImagingControl1.DeviceValid)
            {
                icImagingControl1.LiveStop();
            }
        }

        /// <summary>
        /// ライブディスプレイのサイズをコントロールサイズに調整します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            if (m_bFitImageToWindow)
            {
                if (icImagingControl1.DeviceValid)
                {
                    icImagingControl1.LiveDisplaySize = icImagingControl1.Size;
                }
            }
        }

        /// <summary>
        /// IC ImagingControlのデバイス選択ダイアログを表示します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdDevice_Click(object sender, EventArgs e)
        {
            OpenNewVideoCaptureDevice();
        }

        /// <summary>
        /// 露光時間、明るさなどのデバイスのプロパティダイアログを表示します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdProperties_Click(object sender, EventArgs e)
        {
            ShowDeviceProperties();
        }

        /// <summary>
        /// ライブストリーミングを開始/停止し、開始/停止ボタンの状態を更新します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdLive_Click(object sender, EventArgs e)
        {
            if (icImagingControl1.LiveVideoRunning)
            {
                StopLiveVideo();
            }
            else
            {
                StartLiveVideo();
            }
        }

        ////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////

        /// <summary>
        /// ライブストリーミングを開始し、開始/停止ボタンの状態を更新します。
        /// </summary>
        private void StartLiveVideo()
        {
            icImagingControl1.LiveStart();
            btnSnapEvent.Enabled = false;

            _FramesToCapture = (int)icImagingControl1.DeviceFrameRate *　int.Parse( txtSecond.Text); // キャプチャーする時間;
            UpdateControls();
        }

        /// <summary>
        /// ライブビデオを停止し、開始/停止ボタンの状態を更新します。
        /// </summary>
        private void StopLiveVideo()
        {
            icImagingControl1.LiveStop();

            UpdateControls();
        }

        private void UpdateControls()
        {
            cmdLive.Enabled = icImagingControl1.DeviceValid;
            cmdProperties.Enabled = icImagingControl1.DeviceValid;
            if (icImagingControl1.LiveVideoRunning)
            {
                cmdLive.Text = "Stop Live";
                btnPrepareFrameBufferQueue.Enabled = true;

                btnMemoryRelease.Enabled = true;

            }
            else
            {
                cmdLive.Text = "Start Live";
                btnPrepareFrameBufferQueue.Enabled  = false;
                btnSnapEvent.Enabled = false;

                btnMemoryRelease.Enabled = false;
                btnSaveBmp.Enabled = false;
                btnSaveClip.Enabled = false;
            }
            
        }

        /// <summary>
        /// IC ImagingControlのデバイス選択ダイアログを表示します。
        /// </summary>
        private void OpenNewVideoCaptureDevice()
        {
            if (icImagingControl1.DeviceValid)
            {
                StopLiveVideo();
            }
            else
            {
                icImagingControl1.Device = "";
            }
            icImagingControl1.ShowDeviceSettingsDialog();
            cmdLive.Enabled = icImagingControl1.DeviceValid;
            cmdProperties.Enabled = icImagingControl1.DeviceValid;
            if (icImagingControl1.DeviceValid)
            {
                // 次回のプログラム起動時に自動的に開くことができるように、
                // 現在使用しているデバイスをファイルに保存します。
                icImagingControl1.SaveDeviceStateToFile("device.xml");
            }
            GetFramerateLabel();

        }

        /// <summary>
        /// 露出、明るさなどを変更するためのデバイスのプロパティダイアログを表示します。
        /// すべての変更はファイル「device.xml」に保存されます。
        /// </summary>
        private void ShowDeviceProperties()
        {
            if (icImagingControl1.DeviceValid)
            {
                icImagingControl1.ShowPropertyDialog();
                icImagingControl1.SaveDeviceStateToFile("device.xml");
            }
        }


        /// <summary>
        /// DeviceLostイベントを処理します。
        /// </summary>
        private void DeviceLost()
        {
            MessageBox.Show("Device Lost!");
            UpdateControls();
        }

        private void icImagingControl1_DeviceLost(object sender, TIS.Imaging.ICImagingControl.DeviceLostEventArgs e)
        {
            // We BeginInvoke this so that showing the modal dialog does not block actually closing the device in IC
            BeginInvoke(new Action(() => DeviceLost()));
        }

        /// <summary>
        /// シンクは、画像をキャプチャするためのバッファをキューに割り当てます。
        /// すでにバッファリストがある場合は、削除されます。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPrepareFrameBufferQueue_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            btnPrepareFrameBufferQueue.Enabled = false;
            if (_bufferlist != null)
            {
                _bufferlist = null;
                GC.Collect();
            }
            _sink.AllocAndQueueBuffers(_FramesToCapture);

           
            while(true)
            {
                label3.Text = "バッファに" + (_sink.OutputQueueSize+1) + "枚保存";
                if (_FramesToCapture <= _sink.OutputQueueSize + 1) break;
            }
            btnSnapEvent.Enabled = true;
            
            Cursor = Cursors.Default;
        }

        /// <summary>
        /// キャプチャされたバッファはシンクから_bufferlistにコピーされます。
        /// 最初のバッファが画像ボックスに表示されます。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSnap_Click(object sender, EventArgs e)
        {
            btnSnapEvent.Enabled = false;
            Cursor = Cursors.WaitCursor;
            

            _bufferlist = _sink.PopAllOutputQueueBuffers();
         
            if (_bufferlist.Length > 0)
            {
                tbImageIndex.Maximum = _FramesToCapture - 1;
                tbImageIndex.Value = 0;
                pictureBox1.Image = _bufferlist[0].CreateBitmapWrap();
                btnSaveClip.Enabled = true;
                btnSaveBmp.Enabled = true;
            }

            btnPrepareFrameBufferQueue.Enabled = true;

            Cursor = Cursors.Default;
        }

        /// <summary>
        /// 画像インデックススライダーの値でインデックス付けされた_bufferlistの画像を表示します。
        /// CreateBitmapWrap（）が使用されるため、画像の参照のみが使用され、画像のコピーは使用されません。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbImageIndex_Scroll(object sender, EventArgs e)
        {
            if (_bufferlist == null)
                return;
            if (tbImageIndex.Value < _bufferlist.Length)
            {
                pictureBox1.Image = _bufferlist[tbImageIndex.Value].CreateBitmapWrap();
                pictureBox1.Update();
            }
        }

        private void btnSaveClip_Click(object sender, EventArgs e)
        {
            if( saveClipDlg.ShowDialog(this) == DialogResult.OK)
            {
                SaveBufferList(saveClipDlg.FileName);
            }
        }

        private void SaveBufferList(String FileName)
        {
            MFTestSharp.H264Writer writer;
            //4,000kbps
            int BITRATE = 4 * 1000000;
            Cursor = Cursors.WaitCursor;
            writer = new MFTestSharp.H264Writer(FileName, _sink.OutputFrameType, (int)icImagingControl1.DeviceFrameRate, BITRATE);
            writer.Begin();

            for (int i = 0; i < _bufferlist.Length; i++)
            {
                writer.Write(_bufferlist[i]);
            }
            writer.End();
            Cursor = Cursors.Default;
        }

        private void BtnSaveBmp_Click(object sender, EventArgs e)
        {
            string fileName;
            Cursor = Cursors.WaitCursor;
            for (int i = 0; i < _bufferlist.Length; i++)
            {

                //デバイスが生成したフレーム番号が格納されます。
                string strFrameNumber = _bufferlist[i].FrameMetadata.DeviceFrameNumber.ToString();

                //（有効な場合）デバイスのタイムスタンプが格納されます。
                // デバイス固有のタイムソースによるタイムスタンプです。
                string strDeviceTimeStamp = _bufferlist[i].FrameMetadata.DeviceTimeStamp.ToString();

                //このフレームの最初のパケットをドライバが受信した時刻が格納されます。
                string strSampleStartTime = _bufferlist[i].FrameMetadata.DriverFrameFirstPacketTime.ToString(@"d\_hh\_mm\_ss\_fff");

                //ドライバがこのフレームに与えたフレーム番号が格納されます。
                string strDriverFrameNumber = _bufferlist[i].FrameMetadata.DriverFrameNumber.ToString();

                fileName = "Image " + strSampleStartTime + ".bmp";
                TIS.Imaging.FrameExtensions.SaveAsBitmap(_bufferlist[i], fileName);
            }
            Cursor = Cursors.Default;
        }


        /// <summary>
        /// 利用可能な物理メモリをラベルに表示する
        /// </summary>
        private void TotalPhysicalMemory()
        {

            Microsoft.VisualBasic.Devices.ComputerInfo info =
                new Microsoft.VisualBasic.Devices.ComputerInfo();
            ulong bcs = 1024 * 1024;
            _availablePhysicalMemory = info.AvailablePhysicalMemory / bcs; //利用可能な物理メモリ
            label.Text = "利用可能物理メモリ:" + _availablePhysicalMemory.ToString("#,0") + "MB";
        }

        private void GetFramerateLabel()
        {
            if (icImagingControl1.DeviceValid)
            { 
                float framerate = icImagingControl1.DeviceFrameRate;
                lblFramerate.Text = "設定フレームレート:" + framerate +"fps";
                TIS.Imaging.VideoFormat CurrentVideoFormat = null;

                CurrentVideoFormat = icImagingControl1.VideoFormatCurrent;
                int size = CurrentVideoFormat.BitsPerPixel / 8 * CurrentVideoFormat.Width * CurrentVideoFormat.Height;
                int bcs = 1024 * 1024;
                float memoryConsumption = icImagingControl1.DeviceFrameRate * float.Parse(txtSecond.Text) * size / bcs;
                label2.Text = "消費するメモリ:" + memoryConsumption.ToString("#,0") + "MB";
                if (memoryConsumption > _availablePhysicalMemory && _availablePhysicalMemory!=0 ) label2.ForeColor = Color.Red;
                else  label2.ForeColor = Color.Black;
            }
        }


        private void TxtSecond_TextChanged(object sender, EventArgs e)
        {
            if (txtSecond.Text == "" | txtSecond.Text == "0") txtSecond.Text = "1";
            GetFramerateLabel();
            _FramesToCapture = (int)icImagingControl1.DeviceFrameRate * int.Parse(txtSecond.Text); // キャプチャーする時間;
        }

        /// <summary>
        /// 数字のみ入力可能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TxtSecond_KeyPress(object sender, KeyPressEventArgs e)
        {
            //バックスペースが押された時は有効（Deleteキーも有効）
            if (e.KeyChar == '\b')
            {
                return;
            }

            //数値0～9以外が押された時はイベントをキャンセルする
            if ((e.KeyChar < '0' || '9' < e.KeyChar))
            {
                e.Handled = true;
            }
        }

        private void BtnMemoryRelease_Click(object sender, EventArgs e)
        {

            _bufferlist = null;
            //ガベージコレクション実行
            GC.Collect();
            pictureBox1.Image = null;
            tbImageIndex.Maximum = 0;
            tbImageIndex.Value = 0;
            pictureBox1.Update();

            if (icImagingControl1.LiveVideoRunning) StopLiveVideo();
           
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            TotalPhysicalMemory();
            GetFramerateLabel();

            label3.Text = "バッファに" + _sink.OutputQueueSize + "枚保存";
            if (_sink.OutputQueueSize>0) label3.Text = "バッファに" + (_sink.OutputQueueSize+1) + "枚保存";
        }
    }
}
